class PasswordsHandbookAdminSettingsService {

    /**
     *
     * @param setting
     * @param value
     * @returns {Promise}
     */
    set(setting, value) {
        return this._createRequest(`settings/${setting}`, {value}, 'PUT');
    }

    // noinspection JSMethodCanBeStatic
    /**
     * Creates an api request
     *
     * @param path
     * @param data
     * @param method
     * @returns {Promise}
     * @private
     */
    async _createRequest(path, data = null, method = 'GET') {
        let options = {method, credentials: 'include'};

        options.headers = new Headers();
        options.headers.append('Accept', `application/json, text/plain, */*`);
        options.headers.append('requesttoken', OC.requestToken);

        if(data) {
            if(method === 'GET') options.method = 'POST';
            options.headers.append('Content-Type', 'application/json');
            options.body = JSON.stringify(data);
        }

        try {
            let url      = OC.generateUrl(`/apps/passwords_handbook/admin/${path}`),
                response = await fetch(new Request(url, options));
            return await PasswordsHandbookAdminSettingsService._processResponse(response);
        } catch(e) {
            console.error('Request failed', e);
            throw e;
        }
    }

    /**
     *
     * @param response
     * @returns {Promise<*>}
     * @private
     */
    static async _processResponse(response) {
        if(!response.ok) {
            console.error(response);
            throw new Error('Api request failed');
        }

        let contentType = response.headers.get('content-type');
        if(contentType && contentType.indexOf('application/json') !== -1) {
            return await response.json();
        } else {
            console.error('Invalid response format', response);
            throw new Error('Invalid response format');
        }
    }
}

class PasswordsHandbookAdminSettings {
    constructor() {
        this._timer = {success: null, error: null};
        this.api = new PasswordsHandbookAdminSettingsService();
    }

    initialize() {
        $('[data-handbook-setting]').on(
            'change',
            (e) => {
                let $target = $(e.target),
                    key     = $target.data('handbookSetting'),
                    value   = $target.val();

                if($target.attr('type') === 'checkbox') {
                    value = $target[0].checked;
                }

                this.api.set(key, value)
                    .then((d) => {
                        this._showMessage('saved', `[data-handbook-setting="${key}"]`);
                        if(d.hasOwnProperty('url')) {
                            document.getElementById('passwords-handbook-url').value = d.url;
                        }
                        if(d.hasOwnProperty('error')) {
                            let el = document.getElementById('passwords-handbook-error');
                            if(d.error === null) {
                                el.style.display = 'none';
                            } else {
                                el.innerText = d.error;
                                el.style.display = 'block';
                            }
                        }
                    })
                    .catch((d) => {
                        this._showMessage('error', `[data-handbook-setting="${key}"]`);
                        if(d.message) OC.Notification.show(PasswordsHandbookAdminSettings._translate(d.message));
                    });
            }
        );
    }

    /**
     * Send a request to the server
     *
     * @param url
     * @param data
     * @param target
     * @param success
     * @private
     */
    _sendRequest(url, data, target, success = 'saved') {

        $.post(url, data)
         .success((d) => {
             if(d.status === 'ok') {
                 this._showMessage(success, target);
             } else {
                 this._showMessage('error', target);
                 OC.Notification.show(PasswordsHandbookAdminSettings._translate(d.message));
             }
         })
         .fail(() => {this._showMessage('error', target);});
    }

    /**
     * Show save success/fail message
     *
     * @param type
     * @param target
     * @private
     */
    _showMessage(type, target) {
        let $el = $('section.passwords').find(target).parents('form').eq(0).find(`h3 .response.${type}`);
        $el.removeClass('active').addClass('active');

        clearTimeout(this._timer[type]);
        this._timer[type] = setTimeout(
            () => {
                $el.removeClass('active');
                if(type === 'error') location.reload();
            },
            1000);
    }

    static _translate(text) {
        return OC.L10N.translate('passwords', text);
    }
}

$(window).on('DOMContentLoaded', () => {
    let PwHandbookSettings = new PasswordsHandbookAdminSettings();
    PwHandbookSettings.initialize();
});